#include <libmp/all.h>
#include <stdio.h>

#include "../intl.h"
#include "../cmd.h"
#include "../log.h"

bool cmd_list(lm_ctx_t *ctx, config_t *config, args_t *args){
  char *arg = NULL;

  args_split(args, "grep");
  args_split(args, "desc");

  args_register(args, "grep");
  args_register(args, "desc");

  if((arg = args_register_check(args)) != NULL){
    error(_("Unknown argument: "FG_BOLD"--%s"FG_RESET", run with "FG_BOLD"--help"FG_RESET" to list the avaliable arguments"), arg);
    return false;
  }

  if(args_get_bool(args, "help")){
    info(_("Listing options for the list command:"));
    printf(_("    "FG_BOLD"--grep"FG_RESET":\tmakes the output \"grepable\"\n"));
    printf(_("    "FG_BOLD"--desc"FG_RESET":\tshow package descriptions\n"));
    return true;
  }

  ssize_t count = 0;
  lm_entry_t ent;

  while(lm_ctx_database_next(ctx, &ent))
    count++;
  lm_ctx_database_next_free(ctx, &ent);

  if(count <= 0){
    if(!args_get_bool(args, "grep"))
      info(_("There no installed packages"));
    return true;
  }

  if(!args_get_bool(args, "grep"))
    info(count > 1 ? _("Listing "FG_BOLD"%d"FG_RESET" packages") : _("Listing "FG_BOLD"%d"FG_RESET" package"), count);

  while(lm_ctx_database_next(ctx, &ent)){
    if(args_get_bool(args, "grep")){
      if(args_get_bool(args, "desc"))
        printf("%s:%s:%s\n", ent.name, ent.version, ent.desc);
      else
        printf("%s:%s\n", ent.name, ent.version);
      continue;
    }

    if(args_get_bool(args, "desc"))
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET": %s\n", ent.name, ent.version, ent.desc);
    else
      printf(FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET"\n", ent.name, ent.version);
  }

  lm_ctx_database_next_free(ctx, &ent);
  return true;
}
