#include <libmp/all.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "../callback.h"
#include "../util.h"
#include "../intl.h"
#include "../cmd.h"
#include "../log.h"

bool cmd_remove(lm_ctx_t *ctx, config_t *config, args_t *args){
  char *arg = NULL;

  args_split(args, "break");
  args_split(args, "yes");

  args_register(args, "break");
  args_register(args, "yes");

  if((arg = args_register_check(args)) != NULL){
    error(_("Unknown argument: "FG_BOLD"--%s"FG_RESET", run with "FG_BOLD"--help"FG_RESET" to list the avaliable arguments"), arg);
    return false;
  }

  if(args_get_bool(args, "help")){
    info(_("Listing options for the remove command:"));
    printf(_("    "FG_BOLD"--break"FG_RESET":\tbreak other package depends\n"));
    printf(_("    "FG_BOLD"--yes"FG_RESET":\tdon't ask for confirmation\n"));
    return true;
  }

  ssize_t count = 0, size = 0, current = 0, save = 0;
  bool ret = false, r = false;
  char ssize[LONGSTR_MAX+3];
  lm_entry_t *pkgs = NULL;
  char *name = NULL;

  for(size_t i = 1; i < args->count; i++){
    if(NULL != args->list[i].name || NULL == (name = args->list[i].value))
      continue;

    if(NULL == pkgs)
      pkgs = malloc(sizeof(lm_entry_t));
    else
      pkgs = realloc(pkgs, sizeof(lm_entry_t)*(count+1));

    lm_entry_init(&pkgs[count]);

    if(!lm_ctx_database_find(ctx, &pkgs[count++], name, NULL)){
      if(lm_error() == LM_ERR_DbSqlNotFound)
        error(_("Package "FG_BOLD"%s"FG_RESET" is not installed"), name);
      else
        error(_("Failed to find "FG_BOLD"%s"FG_RESET": %s"), name, lm_strerror());
      goto end;
    }

    if(!args_get_bool(args, "break") && !lm_ctx_removeable(ctx, &pkgs[count-1])){
      error(_("Cannot remove "FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET": %s"), pkgs[count-1].name, pkgs[count-1].version, lm_strerror());
      goto end;
    }

    size += pkgs[count-1].size;
  }

  if(NULL == pkgs){
    error(_("Nothing to do (no packages to remove)"));
    goto end;
  }

  size_to_human(ssize, size);

  if(count == 1)
    info(_("Following "FG_BOLD"%d"FG_RESET" package will be "FG_BOLD"REMOVED:"FG_RESET), count);
  else
    info(_("Following "FG_BOLD"%d"FG_RESET" packages will be "FG_BOLD"REMOVED:"FG_RESET), count);

  // list all the packages
  for(int i = 0; i < count; i++)
    package_list(&pkgs[i], &save, 0);
  printf("\n");

  info(_("Total of "FG_BOLD"%s"FG_RESET" disk space will be freed"), ssize);
  if(!args_get_bool(args, "yes") && !yesno(_("Continue?"))){
    error(_("Operation cancelled"));
    goto end;
  }

  for(int i = 0; i < count; i++){
    lm_entry_t *cur = &pkgs[i];

    info(_("("FG_BOLD"%d"FG_RESET"/"FG_BOLD"%d"FG_RESET") Removing "FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET),
         ++current, count, cur->name, cur->version);

    r = lm_ctx_remove(ctx, cur, callback_remove, NULL);
    bar_free();

    if(!r){
      error(_("Failed to remove "FG_BOLD"%s"FG_RESET"_"FG_BOLD FG_GREEN"%s"FG_RESET": %s"),
            cur->name, cur->version, lm_strerror());
      goto end;
    }
  }

  if(current == count){
    success(count == 1 ? _("Removed "FG_BOLD"%d"FG_RESET" package") : _("Removed all of the "FG_BOLD"%d"FG_RESET" packages"), count);
    ret = true;
  }

  ret = true;
end:
  if(NULL != pkgs)
    for(int i = 0; i < count; i++)
      lm_entry_free(&pkgs[i]);
  bar_free();
  return ret;
}
